##21. Jan. 2003
library(ts)

##1. Spektraldichte von ARMA Prozessen

spec.arma <- function(ar = NULL, ma = NULL, sigma = 1.0)
  {
      ## Purpose: Plottet die Spektraldichte (auf log-Skala) fuer ARMA Prozesse
      ## -------------------------------------------------------------------------
      ## Arguments: AR- und MA-Koeffizienten, sigma
      ## -------------------------------------------------------------------------
      ## Author: Peter Buehlmann, Date: 18 Dec 2000, 11:56
    lambda <- seq(0,pi,0.01)
    Phi <- rep(1,length(lambda))
    Theta <- rep(1,length(lambda))
    if(length(ar)) {
        for(j in 1:length(ar)){
              Phi <- Phi - ar[j]*exp(-(1i*lambda))^j
            }
      }
    if(length(ma)) {
        for(j in 1:length(ma)){
              Theta <- Theta + ma[j]*exp(-(1i*lambda))^j
            }
      }
    plot(lambda,log((sigma*abs(Theta)/abs(Phi))^2/(2*pi)),type="l",main="ARMA spectrum",xlab="frequen\cy",ylab="log(spectral density)")
  }

##Spektraldichte von ARMA Prozess: variiere die Parameter
spec.arma(ar=c(0.99),ma=seq(0.5,0.3))
spec.arma(ar=c(0.8,-0.5))

##sunspot data
data(sunspots)

par(mfrow=c(1,1))
ts.plot(sunspots)

fit <- ar.yw(ts(sunspots))
fit

#AR-Spektrum fuer sunspot Daten
spec.ar(fit)

#Nullstellen fuer sunspot Daten
#Nullstellen von AR- und MA-Polynom

ns.arma <- function(ar = NULL,ma = NULL)
  {
      ## Purpose: berechnet Nullstellen von AR- und MA-Polynom
      ## -------------------------------------------------------------------------
      ## Arguments: AR- und MA-Koeffizienten
      ## -------------------------------------------------------------------------
      ## Author: Peter Buehlmann, Date: 18 Dec 2000, 11:05
    list(ar=abs(polyroot(c(1,- ar))), ma=abs(polyroot(c(1,ma))))
  }

ns.arma(ar=fit$ar)

#Resultat:
> $ar
 [1] 1.089403 1.099563 1.081002 1.015015 1.091106 1.081002 1.080131 1.015015
 [9] 1.116835 1.080267 1.095476 1.159490 1.080131 1.080267 1.091106 1.089745
[17] 1.095476 1.104369 1.116835 1.115337 1.104369 1.099563 1.115337 1.089745
[25] 1.089403 1.159490 1.854561 1.854561

$ma
numeric(0)

#plotte die ersten 10 Nullstellen
plot(polyroot(c(1,-fit$ar)),ylim=c(-1.4,1.4))
points(exp(-(1i)*seq(0,2*pi,0.01)),type="l")

#Frequenzen, wo ersten 2 Nullstellen ungefaehr bei exp(i \lambda)
#an den Frequenzen mit Index [2],[8]

freq <- Arg(polyroot(c(1,-fit$ar)))[c(4,8)]

spec.ar(fit)
lines(rep(freq[2]/(2*pi),49901),seq(100,50000,1),col=2)


#Periode:
2*pi/freq[2]
[1] 128.4377
#approx. 10.7 Jahre

##Cosinus-Schwingung
x.series <- ts(cos((1:1000)*0.1385071))

fit1 <- ar.yw(x.series)
spec.ar(fit1)

which.max(spec.ar(fit1,plot=F)$spec)
[1] 23

2*pi*spec.ar(fit1,plot=F)$freq[23]
[1] 0.1385071

set.seed(22)
x.series2 <- x.series + rnorm(1000)

fit2 <- ar.yw(x.series2)
spec.ar(fit2)

which.max(spec.ar(fit2,plot=F)$spec)
[1] 23

2*pi*spec.ar(fit2,plot=F)$freq[23]
[1] 0.1385071  #immer noch perfektes Auffinden der Hauptfrequenz


##2. Diskrete Fourier-Tranformation: Wir betrachten die Zerlegung einer
##Reihe x(0), x(1), ... , x(N-1) in harmonische Schwingungen mit den
##Frequenzen 2*pi*j/(N-1), j=0,1,2,...,[(N-1)/2].
f.fourier <- function(x)
  {
      ## Purpose: Discrete Fourier transform given as amplitude and phase.
      ## The fast Fourier transform (FFT) is used
      ## -------------------------------------------------------------------------
      ## Arguments: x (real) series
      ## -------------------------------------------------------------------------
      ## Author: Hans-Ruedi Kuensch, Date: 29 Nov 2000, 14:37
      N <- length(x)
        a <- fft(x)
        freq <- (0:((N-1)/2))*2*pi/(N-1)
        a <- a[1:length(freq)]
        list(freq=freq,ampl=Mod(a),phase=Arg(a))
    }


#1. Cosinus-Schwingung und verrauschte Cosinus-Schwingung
xf <- f.fourier(x.series)  #Fourier-Transformierte von x.series
par(mfrow=c(2,1))
plot(xf$freq,xf$ampl,type="h",xlab="Frequenz", ylab="Amplitude")
abline(h=0)
lines(rep(0.1385071,61),seq(-10,50,1),col=2)  #wahre Hauptfrequenz von x.series
plot(xf$freq,xf$phase,type="h",xlab="Frequenz", ylab="Phase")
abline(h=0)

xf2 <- f.fourier(x.series2)
par(mfrow=c(2,1))
plot(xf2$freq,xf2$ampl,type="h",xlab="Frequenz", ylab="Amplitude")
abline(h=0)
lines(rep(0.1385071,61),seq(-10,50,1),col=2)
plot(xf2$freq,xf2$phase,type="h",xlab="Frequenz", ylab="Phase")
abline(h=0)


#2. Weisses Rauschen
x <- ts(rnorm(256))

xf <- f.fourier(x)  #Fourier-Transformierte von x.series
par(mfrow=c(2,1))
plot(xf$freq,xf$ampl,type="h",xlab="Frequenz", ylab="Amplitude")
abline(h=0)
plot(xf$freq,xf$phase,type="h",xlab="Frequenz", ylab="Phase")
abline(h=0)

#Zusaetzlich: Betrachte Histogramme von Amplitude und Phase
hist(xf$ampl,nclass=12)
hist(xf$phase,breaks=seq(-pi,pi,length=12))

#3.Luchsdaten
data("lynx")
x <- log(lynx)
x <- log(lynx)-mean(log(lynx))

xf <- f.fourier(x)  #Fourier-Transformierte von x.series
par(mfrow=c(2,1))
plot(xf$freq,xf$ampl,type="h",xlab="Frequenz", ylab="Amplitude")
abline(h=0)
plot(xf$freq,xf$phase,type="h",xlab="Frequenz", ylab="Phase")
abline(h=0)

#Periode der am staerksten vertretenen Schwingung
which.max(xf$ampl)
[13]
xf$freq[13]
[1] 0.6672409

#Quasi-Periode
2*pi/0.6672409
[1] 9.416667  #Jahre

#4.Ozeanwellen
ocwave <- scan("http://stat.ethz.ch/Teaching/Datasets/ocwave.dat")
x <- ocwave - mean(ocwave)

xf <- f.fourier(x)  #Fourier-Transformierte von x.series
par(mfrow=c(2,1))
plot(xf$freq,xf$ampl,type="h",xlab="Frequenz", ylab="Amplitude")
abline(h=0)
plot(xf$freq,xf$phase,type="h",xlab="Frequenz", ylab="Phase")
abline(h=0)

#erste und zweite Haelfte des Datensatzes
x1  <- x[1:512]
xf1 <- f.fourier(x1)
plot(xf1$freq,xf1$ampl,type="h",xlab="Frequenz", ylab="Amplitude")
x2  <- x[513:1024]
xf2  <- f.fourier(x2)
plot(xf2$freq,xf2$ampl,type="h",xlab="Frequenz", ylab="Amplitude")

#5.Jaehrliche Minima des Nils von 622 - 1284
nile <- ts(scan("http://stat.ethz.ch/Teaching/Datasets/nilmin.dat"),start=622)
plot(nile)
x <- nile - mean(nile)

xf <- f.fourier(x)
par(mfrow=c(1,1))
plot(xf$freq,xf$ampl,xlab="Frequenz",ylab="Amplitude",type="l")
#lange Quasi-Periode


#6.ARCH
f.arch.sim <- function(n=500,init=200,a)
  {
      ## Purpose: Simulating the Arch(1) process
      ## -------------------------------------------------------------------------
      ## Arguments: n=length of the series, init=length of the initial segment
      ## that is discarded, a= Parameter of the model
      ## -------------------------------------------------------------------------
      ## Author: Hans-Ruedi Kuensch, Date:  5 Feb 97, 16:49
      m <- n+init
        eps <- rnorm(m)
        x <- rep(0,(m+init))
        for (i in (2:m)) x[i] <- sqrt(1+a*x[i-1]^2)*eps[i]
        ts(x[(init+1):m],start=1)
    }

set.seed(22)
x <- f.arch.sim(n=1000,a=0.9)
plot(x)

xf <- f.fourier(x)
par(mfrow=c(1,1))
plot(xf$freq,xf$ampl,xlab="Frequenz",ylab="Amplitude",type="l")
#wie weisses Rauschen


#7.Chaotisches System
f.logist <- function(n,start=runif(1))
  {
      ## Purpose: Computing the iterated logistic map x(t+1)=4*x(t)*(1-x(t))
      ## -------------------------------------------------------------------------
      ## Arguments: start=starting value, length=number of iterates
      ## -------------------------------------------------------------------------
      ## Author: Hans-Ruedi Kuensch, Date: 11 Nov 96, 14:59
      x <- rep(0,n)
        x[1] <- start
        for (i in (2:n)) {x[i] <- 4*x[i-1]*(1-x[i-1])}
        x
    }

x <- f.logist(1024)
x <- ts(x-mean(x))

plot(x)
xf <- f.fourier(x)
par(mfrow=c(1,1))
plot(xf$freq,xf$ampl,xlab="log-Frequenz",ylab="log-Amplitude",type="l")
#wie weisses Rauschen
