#9. Nov. 2002

#A. i.i.d. Variablen und Linearkombination von i.i.d Variablen

n <- 100
set.seed(22)
x <- ts(rnorm(n))
par(mfrow=c(2,2))
acf(x)
acf(x^2)
acf(sign(x))
acf(sin(x))

#Linearkombination von 2 i.i.d. Variablen
n <- 100
set.seed(22)
eps <- rnorm(n+1)

x <- ts(eps[1:n] + 0.9*eps[2:(n+1)])
par(mfrow=c(2,2))
acf(x)
acf(x^2)
acf(sign(x))
acf(sin(x))


#B. Lag-Plots und ACF

#1: Luchs
data("lynx")
plot(lynx)
plot(log(lynx))
acf(lynx)
acf(log(lynx))

lag.plot(log(lynx),lag=2,do.lines=T)

#zum Vergleich
set.seed(22)
x <- ts(rnorm(100)) #100 i.i.d. N(0,1)
lag.plot(x,lag=4)   #evtl. mit "do.lines=F"

x <- ts(sin(pi*seq(0:100)/10))
plot(x)
acf(x)

lag.plot(x,lag=4)   #mit lag=25 demonstrieren

#2: Airlines
airline <- ts(scan("http://stat.ethz.ch/Teaching/Datasets/airline.dat"),start=c(1949,1),freq\=12)
load("airline.RData")
plot(log(airline))
acf(log(airline))
acf(diff(diff(log(airline)),lag=12))
airl.stl <- stl(log(airline),s.window=7)
acf(airl.stl$time.series[,3])
lag.plot(airl.stl$time.series[,3],lag=4)


#3: Ozeanwellen: "The change in the level of ambient noise in the ocean"
# (4 Messungen pro Sekunde), aus Percival& Walden, Spectral Analysis for
# Physical Applications
ocwave <- ts(scan("http://stat.ethz.ch/Teaching/Datasets/ocwave.dat"),start=1,deltat=1)
save(ocwave,file="ocwave.RData")
load("ocwave.RData")
plot(ocwave)
acf(ocwave)
acf(ocwave,lag.max=200)
lag.plot(ocwave[1:200],lag=4,do.lines=T)


#4: AR(p)-Modelle
set.seed(22)
x <- arima.sim(n=200,model = list(ar= 0.9)) #Simulation von AR(1) mit phi=0.9
                                            #und eps von N(0,1)

plot(x)
acf(x)
lag.plot(x,lag=4,do.lines=T)


my.rt3 <- function(n)
  {
    rt(n,3)
  }

set.seed(22)
x.t3 <- arima.sim(n=200,model= list(ar=c(0.9)),rand.gen = my.rt3)
plot(x.t3)
acf(x)
lag.plot(x,lag=4,do.lines=T)

par(mfrow=c(2,1))
hist(x,xlim=c(-12,12))
hist(x.t3,xlim=c(-12,12))

set.seed(22)
x.ar2 <- arima.sim(n=200,model=list(ar=c(0.8,-0.7)))
par(mfrow=c(1,1))
plot(x.ar2)
acf(x.ar2)

lag.plot(x.ar2,lag=4,do.lines=T)

#5: AR(1) + ARCH(1)-Innovation

ararch.sim <- function(n,phi,alpha0,alpha1,n.start=100)
{
  ## Purpose: X_t = phi*X_{t-1} + eta_t, eta_t = sigma_t*epsilon_t,
  ##                        sigma_t^2 = alpha_0 + alpha_1 eta_{t-1}^2 
  ## ----------------------------------------------------------------------
  ## Arguments:
  ## ----------------------------------------------------------------------
  ## Author: Prof. P. Buehlmann, Date:  8 Nov 2004, 22:00
epsilon <- rnorm(n+n.start)
eta <- rep(0,n+n.start)
for(i in 2:(n+n.start))
  {
    eta[i] <- sqrt(alpha0 + alpha1*eta[i-1]^2)*epsilon[i]
    x[i] <- phi*x[i-1]+eta[i]
  }
ts(x[(n.start+1):(n+n.start)],start=1,delta=1)
}

set.seed(22)
par(mfrow=c(3,3))
for(i in 1:5)
  {
    x.ararch <- ararch.sim(500,0.7,1,0.8)
    plot(x.ararch)
  }
for(i in 1:4)
  {
    x.ararch <- ararch.sim(500,0.7,1,0)  #AR(1)-Modell
    plot(x.ararch,col=2)
  }

par(mfrow=c(1,1))
set.seed(22)
lag.plot(ararch.sim(200,0.7,1,0.8),lag=4,do.lines=T)


#6: Deterministisches chaotisches System
f.logist <- function(n,start=runif(1))
{
  ## Purpose: Computing the iterated logistic map x(t+1)=4*x(t)*(1-x(t))
  ## -------------------------------------------------------------------------
  ## Arguments: start=starting value, length=number of iterates
  ## -------------------------------------------------------------------------
  ## Author: Hans-Ruedi Kuensch, Date: 11 Nov 96, 14:59
  x <- rep(0,n)
  x[1] <- start
  for (i in (2:n)) {x[i] <- 4*x[i-1]*(1-x[i-1])}
  x
}

x.det <- ts(f.logist(100))
plot(x.det)

acf(x.det)
acf(x.det^7)
lag.plot(x.det,lag=4,do.lines=F)

#6. BMW Daten
bmw <- ts(scan("ftp://ftp.stat.math.ethz.ch/Teaching/buhlmann/Zeitreihen/bmw.dat"))
load("bmw.RData")
par(mfrow=c(1,1))
plot(bmw,main="Tages-Returns von BMW Aktie")

par(mfrow=c(1,2))
acf(bmw)
acf(bmw^2)

lag.plot(bmw^2,lag=4,do.lines=T)
     

