#30.11.2002

#Nullstellen von AR- und MA-Polynom
ns.arma <- function(ar = NULL,ma = NULL)
{
  ## Purpose: berechnet Nullstellen von AR- und MA-Polynom
  ## -------------------------------------------------------------------------
  ## Arguments: AR- und MA-Koeffizienten
  ## -------------------------------------------------------------------------
  ## Author: Peter Buehlmann, Date: 18 Dec 2000, 11:05
list(ar=abs(polyroot(c(1,- ar))), ma=abs(polyroot(c(1,ma))))
}

ns.arma(ar=c(0.8,-0.5),ma=c(0.5,0.3))

#Resultat
> $ar
[1] 1.414214 1.414214

$ma
[1] 1.825742 1.825742

set.seed(22)
x <- arima.sim(1000,model=list(ar=c(0.8,-0.5),ma=c(0.5,0.3)))

#Plotte die Zeitreihe und die acf
par(mfrow=c(2,1))

plot(x)
acf(x)
#Variiere die Parameter


##2. Vergleich von AR(1),MA(1) mit gleicher lag 1 Korrelation von 0.5
x1.sim <- arima.sim(n=1000,model=list(ar=0.5))
x2.sim <- arima.sim(n=1000,model=list(ma=1))
par(mfrow=c(2,1))
upp <- 300
low <- 1
plot(as.ts(x1.sim[low:upp]))
plot(as.ts(x2.sim[low:upp]))

#acf
acf(x1.sim,lag.max=12)
acf(x2.sim,lag.max=12)

#partial acf
pacf(x1.sim,lag.max=12)
pacf(x2.sim,lag.max=12)

#lag.plots
lag.plot(x1.sim,lags=4,do.lines=T)
lag.plot(x2.sim,lags=4,do.lines=T)

##3. Analyse des Ozean-Wellen Datensatzes

ocwave <- ts(scan("http://stat.ethz.ch/Teaching/Datasets/ocwave.dat"))
load("ocwave.RData")
par(mfrow=c(1,1))
plot(ocwave)

#acf
par(mfrow=c(1,1))
acf(ocwave,lag.max=50)

pacf(ocwave,lag.max=50)

#Fitten von ARMA-Modell
fit <- arima(ocwave,c(5,0,2))   #Anpassen von ARMA(5,2)

#AIC-Statistik
res.aic <- matrix(0,nrow=6,ncol=6)
for(i in 1:6)
  {
    for(j in 1:6)
      {
       	fit <- arima(ocwave,c(i,0,j))
        res.aic[i,j] <- fit$aic
        print(c(i,j,res.aic[i,j]))
      }
  }

#dauert ca. 4 Minuten
save(res.aic,file="woche5.RData")

load("woche5.RData")

res.aic

min(res.aic)
[1] 7937.776
#Minimum bei p=6,q=3

fit <- arima(ocwave,c(6,0,3))

ns.arma(ar=fit$coef[1:6],ma=fit$coef[7:9])

#Residuen-Analyse
par(mfrow=c(1,1))
plot(fit$resid)
lines(lowess(1:length(fit$resid),fit$resid),col=2)
qqnorm(fit$resid)
acf(fit$resid)

#acf in simuliertem Modell und Vergleich mit echten Daten
x.sim <- arima.sim(n=1000000,model=list(ar=fit$coef[1:6],ma=fit$coef[7:9]))

acf(ocwave,lag.max=50)
points(0:50,acf(x.sim,lag.max=50,plot=F)$acf,col=2)

#Vorhersage
pred <- predict(fit,n.ahead=100)
par(mfrow=c(1,1))
plot(1:1024,ocwave,xlim=c(1,1124),type="l")
lines(1025:1124,pred$pred,col=2)


#1-step ahead predictions (mit aktuellstem Modell)
pred <- 1:24
for(i in 1:24)
  {
    print(i)
    fit <- arima(ocwave[1:(999+i)],c(6,0,3))
    pred[i] <- predict(fit,n.ahead=1)$pred
  }
#dauert ca. 6 Minuten

save(res.aic,pred,file="woche5.RData")

load("woche5.RData")
plot(ocwave[1001:1024])
points(pred,col=2)

fit <- arima(ocwave[1:1000],c(6,0,3))
pred.glob <- predict(fit,n.ahead=24)
points(1:24,pred.glob$pred,col=3)

#Residuen der 1-Schritt Vorhersage 
plot(ocwave[1001:1024] - pred)

