#03.12.2002

library(ts)

#1. Auto- und parzielle Autokorrelation

#Simulation von ARMA Prozessen (wie in serie5.R)
arma.sim <- function(n, ar = NULL, ma = NULL, sigma = 1.0, innov = NULL)
  {
        x <- ts(rnorm(n+100, 0, sigma), start = -99)
            if(length(innov)) {
                    x  <- ts(innov,start= -99)
                  }
            if(length(ma)) x <- filter(x, c(1, ma), sides=1)
            if(length(ar)) x <- filter(x, ar, method = "recursive")
            as.ts(x[-(1:100)])
      }

set.seed(22)
x.1.0 <- arma.sim(1000,ar=0.8)
x.0.1 <- arma.sim(1000,ma=0.8)
x.1.1 <- arma.sim(1000,ar=0.8,ma=0.8)

par(mfrow=c(1,2))
acf(x.1.0)
acf(x.1.0,type="partial")

acf(x.0.1)
acf(x.0.1,type="partial")

acf(x.1.1)
acf(x.1.1,type="partial")

#Ozeanwellen-Daten
ocwave <- ts(scan("http://stat.ethz.ch/Teaching/Datasets/ocwave.dat"))
load("ocwave.RData")
par(mfrow=c(1,1))
plot(ocwave)

par(mfrow=c(1,2))
acf(ocwave)
acf(ocwave,type="partial")
#ist AR- oder MA angebracht?

fit <- ar.yw(ocwave)  #AR(6) wird angepasst
coef.ocean <- fit$ar  #AR-Koeffizienten

#einfachste Art um Autokorrelationen in AR(6) mit Koeffizienten "coef" zu
#berechnen: SIMULATION!
x.simul <- arma.sim(ar=coef.ocean,n=1000000,sigma=sqrt(797.2))
acf(ocwave)
points(0:30,acf(x.simul,lag.max=30,plot=F)$acf,col=2)
acf(ocwave,type="partial")
points(1:30,acf(x.simul,lag.max=30,type="partial",plot=F)$acf,col=2)


#Luchs-Daten
data(lynx)
loglynx <- log(lynx)
par(mfrow=c(1,1))
plot(loglynx)

par(mfrow=c(1,2))
acf(loglynx)
acf(loglynx,type="partial")
#ist AR- oder MA angebracht?


fit <- ar.yw(loglynx)  #AR(11) wird angepasst
coef.lynx <- fit$ar  #AR-Koeffizienten

#einfachste Art um Autokorrelationen in AR(11) mit Koeffizienten "coef" zu
#berechnen: SIMULATION!
x.simul <- arma.sim(ar=coef.lynx,n=1000000,sigma=sqrt(0.2530))
acf(loglynx)
points(0:30,acf(x.simul,lag.max=30,plot=F)$acf,col=2)
acf(loglynx,type="partial")
points(1:30,acf(x.simul,lag.max=30,type="partial",plot=F)$acf,col=2)


#8 Simulationen und Daten

#Ozeanwellen
x.simul <- matrix(0,ncol=1024,nrow=10)
for(i in 1:8)
  {
        x.simul[i,] <- mean(ocwave) + arma.sim(ar=coef.ocean,n=1024,sigma=sqrt(797.2))
      }

par(mfrow=c(3,3))
for(i in 1:8)
  {
        plot(x.simul[i,],ylim=c(-1000,1600),type="l")
      }
plot(ocwave,ylim=c(-1000,1600),type="l")


#Luchsdaten
x.simul <- matrix(0,ncol=114,nrow=10)
for(i in 1:8)
  {
        x.simul[i,] <- mean(loglynx) + arma.sim(ar=coef.lynx,n=114,sigma=sqrt(0.2530))
      }

par(mfrow=c(3,3))
for(i in 1:8)
  {
        plot(x.simul[i,],ylim=c(3,10),type="l")
      }
plot(loglynx,ylim=c(3,10),type="l")


#2. Vorhersagen

#simuliertes ARMA(1,1)
set.seed(22)
x <- arma.sim(ar=0.8,ma=0.7,n=1100)

xtraining <- as.ts(x[1:1000])
fit <- arima(xtraining,order=c(1,0,1))
#fitted ein ARMA-Modell; hier ein ARMA(1,1)

pred <- predict(fit,n.ahead=100) #Vorhersage-Funktion

par(mfrow=c(1,1))
plot(1001:1100,x[1001:1100],type="l",ylim=c(-6,6))
lines(1001:1100,pred$pred,col=2)
lines(1001:1100,pred$pred + 1.96 * pred$se,col=3)
lines(1001:1100,pred$pred - 1.96 * pred$se,col=3)


#Sonnenflecken
data(sunspots)
par(mfrow=c(1,2))
acf(sunspots,lag.max=30)
acf(sunspots,type="partial",lag.max=30)

fit <- ar.yw(sunspots)

pred <- predict(fit,n.ahead=300)

par(mfrow=c(1,1))
plot(1:2820,sunspots,xlim=c(1,3120),type="l")
lines(2821:3120,pred$pred,col=2)
lines(2821:3120,pred$pred + 1.96 * pred$se,col=3)
lines(2821:3120,pred$pred - 1.96 * pred$se,col=3)


#Ozeanwellen
fit <- arima(ocwave,order=c(6,0,5))  #Ordnung 6,5 mit AIC-Statistik
pred <- predict(fit,n.ahead=100)

par(mfrow=c(1,1))
plot(1:1024,ocwave,xlim=c(1,1124),type="l")
lines(1025:1124,pred$pred,col=2)
lines(1025:1124,pred$pred + 1.96 * pred$se,col=3)
lines(1025:1124,pred$pred - 1.96 * pred$se,col=3)


#1-step ahead predictions (mit aktuellstem Modell)
pred.m <- 1:24
pred.s <- 1:24
for(i in 1:24)
  {
            print(i)
                        fit <- arima(ocwave[1:(999+i)],c(6,0,5))
                        #fit <- ar.yw(ocwave[1:(999+i)])
                        pred.m[i] <- predict(fit,n.ahead=1)$pred
                        pred.s[i] <- predict(fit,n.ahead=1)$se
          }



save(pred.m,pred.s,file="woche6.RData")
load("woche6.RData")

plot(as.ts(ocwave[1001:1024]),ylim=c(-300,650),type="p")
lines(pred.m,col=2)
lines(pred.m-1.96*pred.s,col=3)
lines(pred.m+1.96*pred.s,col=3)

#Anzahl "Verletzungen"
mean(ocwave[1001:1024]<pred.m-1.96*pred.s)
[1] 0.04166667
mean(ocwave[1001:1024]>pred.m+1.96*pred.s)
[1] 0

#Residuen der Vorhersage
plot(ocwave[1001:1024] - pred.m)


#Konzentrations-Messungen (Chemie)
conc <- ts(scan("http://stat.ethz.ch/Teaching/Datasets/konzentration.dat"))
save(conc,file="conc.RData")
load("conc.RData")

fit <- ar.yw(conc)
pred <- predict(fit,n.ahead=100)

par(mfrow=c(1,1))
plot(1:197,conc,xlim=c(1,297),type="l")
lines(198:297,pred$pred,col=2)
lines(198:297,pred$pred + 1.96 * pred$se,col=3)
lines(198:297,pred$pred - 1.96 * pred$se,col=3)

