##10. Dez. 2002
library(ts)

##1. Parameterschaetzungen und Modellwahl

#chemische Konzentrationen
conc <- ts(scan("http://stat.ethz.ch/Teaching/Datasets/konzentration.dat"))
load("conc.RData")
par(mfrow=c(1,1))
plot(conc)

par(mfrow=c(1,2))
acf(conc)
acf(conc,type="partial")

#Yule-Walker fuer Schaetzung von AR
fit.yw <- ar.yw(conc)

fit.yw1 <- ar.yw(conc,order=3)

fit.yw

fit.yw$x.mean

fit.yw$asy.var.coef

par(mfrow=c(1,1))
image(1:7,1:7,fit.yw$asy.var.coef,xlim=c(0.5,7.5),ylim=c(0.5,7.5),col=heat.colors(128))


#MLE fuer Gauss'sche Innovationen
fit.mle <- arima(conc,order=c(7,0,0))

fit.mle

plot(fit.yw$ar,ylab="",main="geschaetzte Koeffizienten")
points(1:7,fit.mle$coef[1:7],col=2)

fit.yw$x.mean

fit.mle$coef[8]


#Kovarianz-Matrix der Schaetzungen
image(1:7,1:7,fit.mle$var.coef[1:7,1:7],xlim=c(0.5,7.5),ylim=c(0.5,7.5),col=heat.colors(128))

range(fit.yw$asy.var.coef-fit.mle$var.coef[1:7,1:7])
[1] -0.0001762586  0.0003661427  #ziemlich aehnliche Kovarianz-Matrizen

image(1:7,1:7,(fit.yw$asy.var.coef-fit.mle$var.coef[1:7,1:7]),xlim=c(0.5,7.5),ylim=c(0.5,7.5),col=heat.colors(128))

#Kleinste Quadrate im Regressionskontext
y <- conc[8:length(conc)]-mean(conc)  #Zielvariablen

x.mat <- matrix(0,ncol=7,nrow=(length(conc)-7))
                                        #Matrix der erklaerenden Variablen
for(i in 1:7)
  {
    x.mat[,i] <- conc[(8-i):(length(conc)-i)]-mean(conc)
  }


fit.lm <- lm(y~x.mat-1)

plot(fit.yw$ar,ylab="",main="geschaetzte Koeffizienten")
points(fit.mle$coef[1:7],col=2)
points(fit.lm$coef,col=3)
legend(5,0.3,c("Yule-Walker","MLE","lineares Modell"),col=1:3,pch=rep(1,3))

summary(fit.lm)

      Coefficients:
         Estimate Std. Error t value Pr(>|t|)
x.mat1  0.34578    0.07256   4.766 3.82e-06 ***
  x.mat2  0.20223    0.07699   2.627  0.00936 **
  x.mat3  0.02788    0.07792   0.358  0.72089
x.mat4  0.01628    0.07692   0.212  0.83257
x.mat5 -0.02044    0.07669  -0.266  0.79016
x.mat6  0.07935    0.07625   1.041  0.29942
x.mat7  0.18880    0.07265   2.599  0.01012 *


  #t-tests mit Yule-Walker
  fit.yw$ar/sqrt(diag(fit.yw$asy.var.coef))

[1]  5.1863758  2.5677082  0.2609748  0.1763027 -0.1903676  0.8141036  2.1752046

#Differenz der P-Werte mit "lm" und "ar.yw"

range(fit.yw$ar/sqrt(diag(fit.yw$asy.var.coef)) - summary(fit.lm)$coef[,3])
[1] -0.4236427  0.4205800

range(fit.yw$asy.var.coef-summary.lm(fit.lm,correlation=T)$cov.unscaled)
[1] -0.05867411  0.01883552


image(1:7,1:7,fit.yw$asy.var.coef-summary.lm(fit.lm,correlation=T)$cov.unscaled,xlim=c(0.5,7.5)\,ylim=c(0.5,7.5),col=heat.colors(128))

#Fazit: Punktschaetzungen mit "lm" OK, aber Genauigkeitsmasse stimmen nicht!


#subset model mit lags 1,2,7

x.mat1 <- matrix(0,ncol=3,nrow=(length(conc)-7))
x.mat1[,1] <- conc[7:(length(conc)-1)]-mean(conc)
x.mat1[,2] <- conc[6:(length(conc)-2)]-mean(conc)
x.mat1[,3] <- conc[1:(length(conc)-7)]-mean(conc)

fit.lm1 <- lm(y~x.mat1-1)

plot(fit.yw$ar,ylab="",main="geschaetzte Koeffizienten")
points(fit.mle$coef[1:7],col=2)
points(fit.lm$coef,col=3)
points(c(1,2,7),fit.lm1$coef,col=4)
legend(4.5,0.35,c("Yule-Walker","MLE","lineares Modell","subset lineares Modell"),col=1:4,pch=rep(1,4))

#Residuen-Analyse
resid <- fit.yw$resid[-(1:7)]
par(mfrow=c(1,2))
acf(resid)
acf(resid^2)
acf(resid,type="partial")
acf(resid^2,type="partial")

par(mfrow=c(1,1))
qqnorm(resid)


resid1 <- fit.lm1$resid
par(mfrow=c(1,2))
acf(resid1)
acf(resid1^2)
acf(resid1,type="partial")
acf(resid1^2,type="partial")

par(mfrow=c(1,1))
qqnorm(resid1)

#AIC-Statistik fuer volles Modell
n <- length(y)

n + n*log(2*pi) + n*log(mean(resid^2)) + 2*8  #AIC-Statistik
[1] 98.9024

#AIC-Statistik fuer subset-Modell
n + n*log(2*pi) + n*log(mean(resid1^2)) + 2*4
[1] 91.94935

#--> subset-Modell ist besser bezueglich AIC


##2. AR mit N(0,1) und t_{nu} Innovationen: Schaetzungne und Modell-Wahl

#Simulation von ARMA Prozessen (wie in serie5.R)
arma.sim <- function(n, ar = NULL, ma = NULL, sigma = 1.0, innov = NULL)
  {
    x <- ts(rnorm(n+100, 0, sigma), start = -99)
                if(length(innov)) {
                                      x  <- ts(innov,start= -99)
                                    }
                if(length(ma)) x <- filter(x, c(1, ma), sides=1)
                if(length(ar)) x <- filter(x, ar, method = "recursive")
                as.ts(x[-(1:100)])
  }

#Simulation von AR(7) mit N(0,1) Innovationen

set.seed(22)
x <- arma.sim(n=200,ar=c(0.8,-0.7,0.6,0,0.4,-0.3,0.2,-0.1))

plot(x)

fit <- ar.yw(x)

#t-tests
fit$ar/sqrt(diag(fit$asy.var.coef))
[1] 11.5929517 -7.0159867  4.9240203  0.2460955  3.9841423 -2.4676216  0.9319779
[8]  0.7732591 -2.4496055

resid <- fit$resid[-(1:fit$order)]
par(mfrow=c(1,2))
acf(resid)
acf(resid^2)
acf(resid,type="partial")
acf(resid^2,type="partial")

par(mfrow=c(1,1))
qqnorm(resid)


#Simulation von AR(7) mit skalierten t_3 Innovationen

set.seed(22)
x <- arma.sim(n=200,ar=c(0.8,-0.7,0.6,0,0.4,-0.3,0.2,-0.1),innov=rt(300,3)/sqrt(3))

plot(x)

fit <- ar.yw(x)

#t-tests
fit$ar/sqrt(diag(fit$asy.var.coef))
[1] 10.3585179 -7.8815423  5.7592592  0.6988191  2.9770541

resid <- fit$resid[-(1:fit$order)]
par(mfrow=c(1,2))
acf(resid)
acf(resid^2)
acf(resid,type="partial")
acf(resid^2,type="partial")

par(mfrow=c(1,1))
qqnorm(resid)
