##11. Jan. 2005

##1. Spektraldichte von ARMA Prozessen

spec.arma <- function(ar = NULL, ma = NULL, sigma = 1.0)
{
  ## Purpose: Plottet die Spektraldichte (auf log-Skala) fuer ARMA Prozesse
  ## -------------------------------------------------------------------------
  ## Arguments: AR- und MA-Koeffizienten, sigma
  ## -------------------------------------------------------------------------
  ## Author: Peter Buehlmann, Date: 18 Dec 2000, 11:56
lambda <- seq(0,pi,0.01)
Phi <- rep(1,length(lambda))
Theta <- rep(1,length(lambda))
if(length(ar)) {
  for(j in 1:length(ar)){
    Phi <- Phi - ar[j]*exp(-(1i*lambda))^j
  }
}
if(length(ma)) {
  for(j in 1:length(ma)){
    Theta <- Theta + ma[j]*exp(-(1i*lambda))^j
  }
}
plot(lambda,log((sigma*abs(Theta)/abs(Phi))^2/(2*pi)),type="l",main="ARMA spectrum",xlab="frequency",ylab="log(spectral density)")
}

##Spektraldichte von ARMA Prozess: variiere die Parameter
spec.arma(ar=c(0.99),ma=seq(0.5,0.3))
spec.arma(ar=c(0.8,-0.5))

##sunspot data
data(sunspots)

plot(sunspots)

fit <- ar.yw(ts(sunspots))
fit

#AR-Spektrum fuer sunspot Daten
spec.ar(fit)

#oder
spectrum(sunspots,method="ar")

#Nullstellen fuer sunspot Daten
#Nullstellen von AR- und MA-Polynom

ns.arma <- function(ar = NULL,ma = NULL)
{
  ## Purpose: berechnet Nullstellen von AR- und MA-Polynom
  ## -------------------------------------------------------------------------
  ## Arguments: AR- und MA-Koeffizienten
  ## -------------------------------------------------------------------------
  ## Author: Peter Buehlmann, Date: 18 Dec 2000, 11:05
list(ar=abs(polyroot(c(1,- ar))), ma=abs(polyroot(c(1,ma))))
}

ns.arma(ar=fit$ar)

#Resultat:
> $ar
 [1] 1.089403 1.099563 1.081002 1.015015 1.091106 1.081002 1.080131 1.015015
 [9] 1.116835 1.080267 1.095476 1.159490 1.080131 1.080267 1.091106 1.089745
[17] 1.095476 1.104369 1.116835 1.115337 1.104369 1.099563 1.115337 1.089745
[25] 1.089403 1.159490 1.854561 1.854561

$ma
numeric(0)

#plotte die ersten 10 Nullstellen
plot(polyroot(c(1,-fit$ar)),ylim=c(-1.4,1.4))
points(exp(-(1i)*seq(0,2*pi,0.01)),type="l")
abline(0,0)

#Frequenzen, wo ersten 2 Nullstellen ungefaehr bei exp(i \lambda)
#an den Frequenzen mit Index [2],[8]

freq <- Arg(polyroot(c(1,-fit$ar)))[c(4,8)]

spec.ar(fit)
lines(rep(freq[2]/(2*pi),49901),seq(100,50000,1),col=2)


#Periode:
2*pi/freq[2]
[1] 128.4377
#approx. 10.7 Jahre

##Cosinus-Schwingung
x.series <- ts(cos((1:1000)*0.1385071))

plot(x.series)

fit1 <- ar.yw(x.series)
spec.ar(fit1)

which.max(spec.ar(fit1,plot=F)$spec)
[1] 23

2*pi*spec.ar(fit1,plot=F)$freq[23]
[1] 0.1385071

set.seed(22)
x.series2 <- x.series + rnorm(1000,0,1)
plot(x.series2)


fit2 <- ar.yw(x.series2)
spec.ar(fit2)

which.max(spec.ar(fit2,plot=F)$spec)
[1] 23

2*pi*spec.ar(fit2,plot=F)$freq[23]
[1] 0.1385071  #immer noch perfektes Auffinden der Hauptfrequenz


##2. Diskrete Fourier-Tranformation: Wir betrachten die Zerlegung einer
##Reihe x(0), x(1), ... , x(N-1) in harmonische Schwingungen mit den
##Frequenzen 2*pi*j/(N-1), j=0,1,2,...,[(N-1)/2].
f.fourier <- function(x)
{
  ## Purpose: Discrete Fourier transform given as amplitude and phase.
  ## The fast Fourier transform (FFT) is used
  ## -------------------------------------------------------------------------
  ## Arguments: x (real) series
  ## -------------------------------------------------------------------------
  ## Author: Hans-Ruedi Kuensch, Date: 29 Nov 2000, 14:37
  N <- length(x)
  a <- fft(x)
  freq <- (0:((N-1)/2))*2*pi/(N-1)
  a <- a[1:length(freq)]
  list(freq=freq,ampl=Mod(a),phase=Arg(a))
}

par(mfrow=c(1,1))
spec1 <- spectrum(x.series)
2*pi*spec1$freq[which.max(spec1$spec)]
[1] 0.1382301   #Hauptfrequenz

par(mfrow=c(1,1))
spec2 <- spectrum(x.series2)
2*pi*spec2$freq[which.max(spec2$spec)]
[1] 0.1382301   #Hauptfrequenz


#weisses Rauschen
x.series3 <- ts(rnorm(256))
spec3 <- spectrum(x.series3)

#Luchsdaten
data("lynx")
plot(lynx)
x.series4 <- log(lynx)

spec4 <- spectrum(x.series4)
2*pi*spec4$freq[which.max(spec4$spec)]
[1] 0.6806784   #Hauptfrequenz
2*pi/0.6806784
[1] 9.23077  #Jahre

spec5 <- spectrum(x.series4,method="ar")
2*pi*spec5$freq[which.max(spec5$spec)]
[1] 0.648465  #Hauptfrequenz
2*pi/0.648465
[1] 9.68932 #Jahre

#Jaehrliche Minima des Nils von 622 - 1284
nile <- ts(scan("http://stat.ethz.ch/Teaching/Datasets/nilmin.dat"),start=622)

save(nile,file="serie10new.RData")
load("serie10new.RData")

plot(nile)

spec6 <- spectrum(nile)
2*pi*spec6$freq[which.max(spec6$spec)]
[1] 0.02792527  #Hauptfrequenz
2*pi/0.02792527
[1] 225

spec7 <- spectrum(nile,method="ar")
2*pi*spec7$freq[which.max(spec7$spec)]
[1] 0

lines(rep(spec7$freq[117],50),seq(1001,6000,100),col=2)

2*pi*spec7$freq[117]
[1] 0.7303101

2*pi/0.7303101
[1] 8.603448     #"Joseph-Effekt"?


#Glaettung des Periodograms:
par(mfrow=c(3,3))
spectrum(nile)
for(i in 2:9)
  {
    spectrum(nile,spans=2*i-1)
  }



